@@ -14,6 +14,8 @@ class TestMultipleNics(base.BaseNamespacedTestCase):
             '192.168.242.0/24', True, True, '%s-net-one' % self.namespace)
         self.net_two = self.test_client.allocate_network(
             '192.168.243.0/24', True, True, '%s-net-two' % self.namespace)
+        self._await_networks_ready([self.net_one['uuid'],
+                                    self.net_two['uuid']])
 
     def test_simple(self):
         ud = """#!/bin/sh
@@ -44,10 +46,14 @@ sudo /etc/init.d/S40network restart"""
 
         self.assertIsNotNone(inst['uuid'])
 
+        self._await_instances_ready([inst['uuid']])
         self._await_login_prompt(inst['uuid'])
 
         ifaces = self.test_client.get_instance_interfaces(inst['uuid'])
         self.assertEqual(2, len(ifaces))
+        for iface in ifaces:
+            self.assertEqual('created', iface['state'],
+                             'Interface %s is not in correct state' % iface['uuid'])
 
         for iface in ifaces:
             self._test_ping(
