@@ -7,9 +7,9 @@ import shutil
 import signal
 
 from shakenfist.config import config
+from shakenfist import instance
 from shakenfist import networkinterface
-from shakenfist import util
-from shakenfist import virt
+from shakenfist.util import process as util_process
 
 
 class DHCP(object):
@@ -18,18 +18,17 @@ class DHCP(object):
 
         self.subst = {
             'config_dir': os.path.join(
-                config.get('STORAGE_PATH'), 'dhcp', self.network.uuid),
-            'zone': config.get('ZONE'),
+                config.STORAGE_PATH, 'dhcp', self.network.uuid),
+            'zone': config.ZONE,
 
             'router': network.router,
             'dhcp_start': network.dhcp_start,
             'netmask': network.netmask,
             'broadcast': network.broadcast,
-            'dns_server': config.get('DNS_SERVER'),
-            'mtu': config.get('MAX_HYPERVISOR_MTU') - 50,
+            'dns_server': config.DNS_SERVER,
+            'mtu': config.MAX_HYPERVISOR_MTU - 50,
+            'provide_nat': network.provide_nat,
 
-            'netns': self.network.uuid,
-            'in_netns': 'ip netns exec %s' % self.network.uuid,
             'interface': interface
         }
 
@@ -40,8 +39,7 @@ class DHCP(object):
         return ('dhcp', self.network.uuid)
 
     def _read_template(self, template):
-        with open(os.path.join(config.get('STORAGE_PATH'),
-                               template)) as f:
+        with open(os.path.join(config.STORAGE_PATH, template)) as f:
             return jinja2.Template(f.read())
 
     def _make_config(self):
@@ -63,8 +61,8 @@ class DHCP(object):
 
         instances = []
         for ni in networkinterface.interfaces_for_network(self.network):
-            instance = virt.Instance.from_db(ni.instance_uuid)
-            if not instance:
+            inst = instance.Instance.from_db(ni.instance_uuid)
+            if not inst:
                 continue
 
             instances.append(
@@ -72,7 +70,7 @@ class DHCP(object):
                     'uuid': ni.instance_uuid,
                     'macaddr': ni.macaddr,
                     'ipv4': ni.ipv4,
-                    'name': instance.name.replace(',', '')
+                    'name': inst.name.replace(',', '')
                 }
             )
         self.subst['instances'] = instances
@@ -108,11 +106,12 @@ class DHCP(object):
         self._remove_config()
 
     def restart_dhcpd(self):
-        if not os.path.exists('/var/run/netns/%(netns)s' % self.subst):
+        if not os.path.exists('/var/run/netns/%s' % self.network.uuid):
             return
 
         self._make_config()
         self._make_hosts()
         if not self._send_signal(signal.SIGHUP):
-            util.execute(None,
-                         '%(in_netns)s dnsmasq --conf-file=%(config_dir)s/config' % self.subst)
+            util_process.execute(
+                None, 'dnsmasq --conf-file=%(config_dir)s/config' % self.subst,
+                namespace=self.network.uuid)
