@@ -20,13 +20,14 @@ from shakenfist.external_api import (
     auth as api_auth,
     base as api_base,
     blob as api_blob,
-    image as api_image,
+    artifact as api_artifact,
     interface as api_interface,
     instance as api_instance,
     label as api_label,
     network as api_network,
     node as api_node,
-    snapshot as api_snapshot)
+    snapshot as api_snapshot,
+    upload as api_upload)
 from shakenfist import logutil
 
 
@@ -114,8 +115,16 @@ api.add_resource(api_interface.InterfaceFloatEndpoint,
 api.add_resource(api_interface.InterfaceDefloatEndpoint,
                  '/interfaces/<interface_uuid>/defloat')
 
-api.add_resource(api_image.ImagesEndpoint, '/images')
-api.add_resource(api_image.ImageEventsEndpoint, '/images/events')
+api.add_resource(api_artifact.ArtifactEndpoint, '/artifacts/<artifact_uuid>')
+api.add_resource(api_artifact.ArtifactsEndpoint, '/artifacts')
+api.add_resource(api_artifact.ArtifactUploadEndpoint,
+                 '/artifacts/upload/<artifact_name>')
+api.add_resource(api_artifact.ArtifactEventsEndpoint,
+                 '/artifacts/<artifact_uuid>/events')
+api.add_resource(api_artifact.ArtifactVersionsEndpoint,
+                 '/artifacts/<artifact_uuid>/versions')
+api.add_resource(api_artifact.ArtifactVersionEndpoint,
+                 '/artifacts/<artifact_uuid>/versions/<version_id>')
 
 api.add_resource(api_label.LabelEndpoint, '/label/<label_name>')
 
@@ -133,3 +142,6 @@ api.add_resource(api_network.NetworkPingEndpoint,
                  '/networks/<network_uuid>/ping/<address>')
 
 api.add_resource(api_node.NodesEndpoint, '/nodes')
+
+api.add_resource(api_upload.UploadCreateEndpoint, '/upload')
+api.add_resource(api_upload.UploadDataEndpoint, '/upload/<upload_uuid>')
