@@ -22,12 +22,13 @@ from shakenfist.config import config
 from shakenfist import db
 from shakenfist import etcd
 from shakenfist import exceptions
-from shakenfist import images
 from shakenfist import logutil
 from shakenfist import net
 from shakenfist import networkinterface
-from shakenfist import util
 from shakenfist.tasks import DeleteInstanceTask
+from shakenfist.util import general as util_general
+from shakenfist.util import image as util_image
+from shakenfist.util import libvirt as util_libvirt
 
 
 LOG, _ = logutil.setup(__name__)
@@ -40,14 +41,20 @@ def _get_defaulted_disk_bus(disk):
     return config.DISK_BUS
 
 
-def _get_disk_device_base(bus):
+LETTERS = 'abcdefghijklmnopqrstuvwxyz'
+NUMBERS = '0123456789'
+
+
+def _get_disk_device(bus, index):
     bases = {
-        'ide': 'hd',
-        'scsi': 'sd',
-        'usb': 'sd',
-        'virtio': 'vd'
+        'ide': ('hd', LETTERS),
+        'scsi': ('sd', LETTERS),
+        'usb': ('sd', LETTERS),
+        'virtio': ('vd', LETTERS),
+        'nvme': ('nvme', NUMBERS),
     }
-    return bases.get(bus, 'sd')
+    prefix, index_scheme = bases.get(bus, 'sd')
+    return '%s%s' % (prefix, index_scheme[index])
 
 
 def _get_defaulted_disk_type(disk):
@@ -65,8 +72,8 @@ def _safe_int_cast(i):
 
 def _initialize_block_devices(instance_path, disk_spec):
     bus = _get_defaulted_disk_bus(disk_spec[0])
-    root_device = _get_disk_device_base(bus) + 'a'
-    config_device = _get_disk_device_base(bus) + 'b'
+    root_device = _get_disk_device(bus, 0)
+    config_device = _get_disk_device(bus, 1)
 
     disk_type = 'qcow2'
 
@@ -91,19 +98,22 @@ def _initialize_block_devices(instance_path, disk_spec):
                 'present_as': 'disk',
                 'snapshot_ignores': True
             }
-        ]
+        ],
+        'extracommands': []
     }
 
-    i = 0
+    i = 2
     for d in disk_spec[1:]:
         bus = _get_defaulted_disk_bus(d)
-        device = _get_disk_device_base(bus) + chr(ord('c') + i)
+        device = _get_disk_device(bus, i)
+        disk_path = os.path.join(instance_path, device)
+
         block_devices['devices'].append({
             'type': disk_type,
             'size': _safe_int_cast(d.get('size')),
             'device': device,
             'bus': bus,
-            'path': os.path.join(instance_path, device),
+            'path': disk_path,
             'base': d.get('base'),
             'blob_uuid': d.get('blob_uuid'),
             'present_as': _get_defaulted_disk_type(d),
@@ -111,16 +121,23 @@ def _initialize_block_devices(instance_path, disk_spec):
         })
         i += 1
 
+    # NVME disks require a different treatment because libvirt doesn't natively
+    # support them yet.
+    nvme_counter = 0
+    for d in block_devices['devices']:
+        if d['bus'] == 'nvme':
+            nvme_counter += 1
+            block_devices['extracommands'].extend([
+                '-drive', ('file=%s,format=%s,if=none,id=NVME%d'
+                           % (d['path'], d['type'], nvme_counter)),
+                '-device', ('nvme,drive=NVME%d,serial=nvme-%d'
+                            % (nvme_counter, nvme_counter))
+            ])
+
     block_devices['finalized'] = False
     return block_devices
 
 
-def _blob_path():
-    blob_path = os.path.join(config.STORAGE_PATH, 'blobs')
-    os.makedirs(blob_path, exist_ok=True)
-    return blob_path
-
-
 class Instance(dbo):
     object_type = 'instance'
     current_version = 3
@@ -131,18 +148,26 @@ class Instance(dbo):
     STATE_PREFLIGHT_ERROR = 'preflight-error'
     STATE_CREATING_ERROR = 'creating-error'
     STATE_CREATED_ERROR = 'created-error'
+    STATE_DELETE_WAIT_ERROR = 'delete-wait-error'
 
     state_targets = {
         None: (dbo.STATE_INITIAL, dbo.STATE_ERROR),
-        dbo.STATE_INITIAL: (STATE_PREFLIGHT, dbo.STATE_DELETED, STATE_INITIAL_ERROR),
-        STATE_PREFLIGHT: (dbo.STATE_CREATING, dbo.STATE_DELETED, STATE_PREFLIGHT_ERROR),
-        dbo.STATE_CREATING: (dbo.STATE_CREATED, dbo.STATE_DELETED, STATE_CREATING_ERROR),
-        dbo.STATE_CREATED: (dbo.STATE_DELETED, STATE_CREATED_ERROR),
+        dbo.STATE_INITIAL: (STATE_PREFLIGHT, dbo.STATE_DELETE_WAIT,
+                            dbo.STATE_DELETED, STATE_INITIAL_ERROR),
+        STATE_PREFLIGHT: (dbo.STATE_CREATING, dbo.STATE_DELETE_WAIT,
+                          dbo.STATE_DELETED, STATE_PREFLIGHT_ERROR),
+        dbo.STATE_CREATING: (dbo.STATE_CREATED, dbo.STATE_DELETE_WAIT,
+                             dbo.STATE_DELETED, STATE_CREATING_ERROR),
+        dbo.STATE_CREATED: (dbo.STATE_DELETE_WAIT, dbo.STATE_DELETED,
+                            STATE_CREATED_ERROR),
         STATE_INITIAL_ERROR: (dbo.STATE_ERROR),
         STATE_PREFLIGHT_ERROR: (dbo.STATE_ERROR),
         STATE_CREATING_ERROR: (dbo.STATE_ERROR),
         STATE_CREATED_ERROR: (dbo.STATE_ERROR),
-        dbo.STATE_ERROR: (dbo.STATE_DELETED, dbo.STATE_ERROR),
+        dbo.STATE_ERROR: (dbo.STATE_DELETE_WAIT, dbo.STATE_DELETED,
+                          dbo.STATE_ERROR),
+        dbo.STATE_DELETE_WAIT: (dbo.STATE_DELETED, STATE_DELETE_WAIT_ERROR),
+        STATE_DELETE_WAIT_ERROR: (dbo.STATE_ERROR),
         dbo.STATE_DELETED: None,
     }
 
@@ -195,7 +220,6 @@ class Instance(dbo):
         i.state = cls.STATE_INITIAL
         i._db_set_attribute(
             'power_state', {'power_state': cls.STATE_INITIAL})
-        i.add_event('db record creation', None)
         return i
 
     @staticmethod
@@ -237,11 +261,12 @@ class Instance(dbo):
             'power_state',
             'vdi_port'
         ]
+
         # Ensure that missing attributes still get reported
         for attr in external_attribute_key_whitelist:
             i[attr] = None
 
-        for attrname in ['placement', 'state', 'power_state', 'ports']:
+        for attrname in ['placement', 'power_state', 'ports']:
             d = self._db_get_attribute(attrname)
             for key in d:
                 if key not in external_attribute_key_whitelist:
@@ -253,6 +278,17 @@ class Instance(dbo):
 
                 i[key] = d[key]
 
+        # Mix in details of the instance's interfaces to reduce API round trips
+        # for clients.
+        i['interfaces'] = []
+        for iface_uuid in self.interfaces:
+            ni = networkinterface.NetworkInterface.from_db(iface_uuid)
+            if not ni:
+                self.log.with_fields({'networkinterface': iface_uuid}).error(
+                    'Network interface missing')
+            else:
+                i['interfaces'].append(ni.external_view())
+
         return i
 
     # Static values
@@ -325,6 +361,14 @@ class Instance(dbo):
     def block_devices(self):
         return self._db_get_attribute('block_devices')
 
+    @property
+    def interfaces(self):
+        return self._db_get_attribute('interfaces')
+
+    @interfaces.setter
+    def interfaces(self, interfaces):
+        self._db_set_attribute('interfaces', interfaces)
+
     # Implementation
     def place_instance(self, location):
         with self.get_lock_attr('placement', 'Instance placement'):
@@ -374,8 +418,9 @@ class Instance(dbo):
     # creation. It is assumed that the image sits in local cache already, and
     # has been transcoded to the right format. This has been done to facilitate
     # moving to a queue and task based creation mechanism.
-    def create(self, lock=None):
+    def create(self, iface_uuids, lock=None):
         self.state = self.STATE_CREATING
+        self.interfaces = iface_uuids
 
         # Ensure we have state on disk
         os.makedirs(self.instance_path, exist_ok=True)
@@ -386,7 +431,7 @@ class Instance(dbo):
         # Create the actual instance. Sometimes on Ubuntu 20.04 we need to wait
         # for port binding to work. Revisiting this is tracked by issue 320 on
         # github.
-        with util.RecordedOperation('create domain', self):
+        with util_general.RecordedOperation('create domain', self):
             if not self.power_on():
                 attempts = 0
                 while not self.power_on() and attempts < 5:
@@ -403,7 +448,7 @@ class Instance(dbo):
             self.enqueue_delete_due_error('Instance failed to power on')
 
     def delete(self):
-        with util.RecordedOperation('delete domain', self):
+        with util_general.RecordedOperation('delete domain', self):
             try:
                 self.power_off()
 
@@ -417,14 +462,14 @@ class Instance(dbo):
                 if inst:
                     inst.undefine()
             except Exception as e:
-                util.ignore_exception('instance delete', e)
+                util_general.ignore_exception('instance delete', e)
 
-        with util.RecordedOperation('delete disks', self):
+        with util_general.RecordedOperation('delete disks', self):
             try:
                 if os.path.exists(self.instance_path):
                     shutil.rmtree(self.instance_path)
             except Exception as e:
-                util.ignore_exception('instance delete', e)
+                util_general.ignore_exception('instance delete', e)
 
         self.deallocate_instance_ports()
 
@@ -499,7 +544,7 @@ class Instance(dbo):
                                                           self.disk_spec)
 
             # Generate a config drive
-            with util.RecordedOperation('make config drive', self):
+            with util_general.RecordedOperation('make config drive', self):
                 self._make_config_drive(
                     os.path.join(self.instance_path,
                                  block_devices['devices'][1]['path']))
@@ -531,10 +576,12 @@ class Instance(dbo):
                         disk_base = '%s%s' % (BLOB_URL, disk['blob_uuid'])
 
                     if disk_base:
-                        cached_image_path = os.path.join(
-                            config.STORAGE_PATH, 'image_cache', disk['blob_uuid'] + '.qcow2')
+                        cached_image_path = util_general.file_permutation_exists(
+                            os.path.join(config.STORAGE_PATH,
+                                         'image_cache', disk['blob_uuid']),
+                            ['iso', 'qcow2'])
 
-                        with util.RecordedOperation('detect cdrom images', self):
+                        with util_general.RecordedOperation('detect cdrom images', self):
                             try:
                                 cd = pycdlib.PyCdlib()
                                 cd.open(cached_image_path)
@@ -548,22 +595,30 @@ class Instance(dbo):
                                 '.qcow2', '.raw')
                             disk['type'] = 'raw'
                             disk['snapshot_ignores'] = True
+                            util_general.link(cached_image_path, disk['path'])
 
-                            try:
-                                os.link(cached_image_path, disk['path'])
-                            except OSError:
-                                os.symlink(cached_image_path, disk['path'])
+                            # qemu does not support removable media on virtio buses. It also
+                            # only supports one IDE bus. This is quite limiting. Instead, we
+                            # use USB for cdrom drives, unless you've specified a bus other
+                            # than virtio in the creation request.
+                            if disk['bus'] == 'virtio':
+                                disk['bus'] = 'usb'
+                                disk['device'] = _get_disk_device(
+                                    disk['bus'], LETTERS.find(disk['device'][-1]))
+
+                        elif disk['bus'] == 'nvme':
+                            # NVMe disks do not currently support a COW layer for the instance
+                            # disk. This is because we don't have a libvirt <disk/> element for
+                            # them and therefore can't specify their backing store. Instead we
+                            # produce a flat layer here.
+                            util_image.create_qcow2([lock], cached_image_path,
+                                                    disk['path'], disk_size=disk['size'])
 
-                            # Due to limitations in some installers, cdroms are always on IDE
-                            disk['device'] = 'hd%s' % disk['device'][-1]
-                            disk['bus'] = 'ide'
                         else:
-                            with util.RecordedOperation('create copy on write layer', self):
-                                images.create_cow([lock], cached_image_path,
-                                                  disk['path'], disk['size'])
-                            shutil.chown(
-                                disk['path'], 'libvirt-qemu', 'libvirt-qemu')
-                            self.log.with_fields(util.stat_log_fields(disk['path'])).info(
+                            with util_general.RecordedOperation('create copy on write layer', self):
+                                util_image.create_cow([lock], cached_image_path,
+                                                      disk['path'], disk['size'])
+                            self.log.with_fields(util_general.stat_log_fields(disk['path'])).info(
                                 'COW layer %s created' % disk['path'])
 
                             # Record the backing store for modern libvirts
@@ -575,8 +630,10 @@ class Instance(dbo):
                                 % (cached_image_path))
 
                     elif not os.path.exists(disk['path']):
-                        images.create_blank([lock], disk['path'], disk['size'])
+                        util_image.create_blank(
+                            [lock], disk['path'], disk['size'])
 
+                    shutil.chown(disk['path'], 'libvirt-qemu', 'libvirt-qemu')
                     modified_disks.append(disk)
 
                 block_devices['devices'] = modified_disks
@@ -648,7 +705,8 @@ class Instance(dbo):
         }
 
         have_default_route = False
-        for iface in networkinterface.interfaces_for_instance(self):
+        for iface_uuid in self.interfaces:
+            iface = networkinterface.NetworkInterface.from_db(iface_uuid)
             if iface.ipv4:
                 devname = 'eth%d' % iface.order
                 nd['links'].append(
@@ -731,13 +789,15 @@ class Instance(dbo):
             t = jinja2.Template(f.read())
 
         networks = []
-        for ni in networkinterface.interfaces_for_instance(self):
+        for iface_uuid in self.interfaces:
+            ni = networkinterface.NetworkInterface.from_db(iface_uuid)
             n = net.Network.from_db(ni.network_uuid)
             networks.append(
                 {
                     'macaddr': ni.macaddr,
                     'bridge': n.subst_dict()['vx_bridge'],
-                    'model': ni.model
+                    'model': ni.model,
+                    'mtu': config.MAX_HYPERVISOR_MTU - 50
                 }
             )
 
@@ -746,7 +806,7 @@ class Instance(dbo):
         # I hadn't spent _ages_ finding a bug related to it.
         block_devices = self.block_devices
         ports = self.ports
-        return t.render(
+        x = t.render(
             uuid=self.uuid,
             memory=self.memory * 1024,
             vcpus=self.cpus,
@@ -757,11 +817,21 @@ class Instance(dbo):
             vdi_port=ports.get('vdi_port'),
             video_model=self.video['model'],
             video_memory=self.video['memory'],
-            uefi=self.uefi
+            uefi=self.uefi,
+            extracommands=block_devices.get('extracommands', [])
         )
 
+        # Libvirt re-writes the domain XML once loaded, so we store the XML
+        # as generated as well so that we can debug. Note that this is _not_
+        # the XML actually used by libvirt.
+        os.makedirs(self.instance_path, exist_ok=True)
+        with open(os.path.join(self.instance_path, 'original_domain.xml'), 'w') as f:
+            f.write(x)
+
+        return x
+
     def _get_domain(self):
-        libvirt = util.get_libvirt()
+        libvirt = util_libvirt.get_libvirt()
         conn = libvirt.open('qemu:///system')
         try:
             return conn.lookupByName('sf:' + self.uuid)
@@ -774,11 +844,11 @@ class Instance(dbo):
         if not inst:
             return 'off'
 
-        libvirt = util.get_libvirt()
-        return util.extract_power_state(libvirt, inst)
+        libvirt = util_libvirt.get_libvirt()
+        return util_libvirt.extract_power_state(libvirt, inst)
 
     def power_on(self):
-        libvirt = util.get_libvirt()
+        libvirt = util_libvirt.get_libvirt()
         inst = self._get_domain()
         if not inst:
             conn = libvirt.open('qemu:///system')
@@ -802,6 +872,15 @@ class Instance(dbo):
                 ports = self.ports
                 self._free_console_port(ports['console_port'])
                 self._free_console_port(ports['vdi_port'])
+
+                # We need to delete the nvram file before we can undefine
+                # the domain. This will be recreated by libvirt on the next
+                # attempt.
+                nvram_path = os.path.join(
+                    config.STORAGE_PATH, 'instances', self.uuid, 'nvram')
+                if os.path.exists(nvram_path):
+                    os.unlink(nvram_path)
+
                 inst.undefine()
 
                 self.ports = None
@@ -812,12 +891,13 @@ class Instance(dbo):
                 return False
 
         inst.setAutostart(1)
-        self.update_power_state(util.extract_power_state(libvirt, inst))
+        self.update_power_state(
+            util_libvirt.extract_power_state(libvirt, inst))
         self.add_event('poweron', 'complete')
         return True
 
     def power_off(self):
-        libvirt = util.get_libvirt()
+        libvirt = util_libvirt.get_libvirt()
         inst = self._get_domain()
         if not inst:
             return
@@ -825,13 +905,15 @@ class Instance(dbo):
         try:
             inst.destroy()
         except libvirt.libvirtError as e:
-            self.log.error('Failed to delete domain: %s', e)
+            if not str(e).startswith('Requested operation is not valid: '
+                                     'domain is not running'):
+                self.log.error('Failed to delete domain: %s', e)
 
         self.update_power_state('off')
         self.add_event('poweroff', 'complete')
 
     def reboot(self, hard=False):
-        libvirt = util.get_libvirt()
+        libvirt = util_libvirt.get_libvirt()
         inst = self._get_domain()
         if not hard:
             inst.reboot(flags=libvirt.VIR_DOMAIN_REBOOT_ACPI_POWER_BTN)
@@ -840,17 +922,19 @@ class Instance(dbo):
         self.add_event('reboot', 'complete')
 
     def pause(self):
-        libvirt = util.get_libvirt()
+        libvirt = util_libvirt.get_libvirt()
         inst = self._get_domain()
         inst.suspend()
-        self.update_power_state(util.extract_power_state(libvirt, inst))
+        self.update_power_state(
+            util_libvirt.extract_power_state(libvirt, inst))
         self.add_event('pause', 'complete')
 
     def unpause(self):
-        libvirt = util.get_libvirt()
+        libvirt = util_libvirt.get_libvirt()
         inst = self._get_domain()
         inst.resume()
-        self.update_power_state(util.extract_power_state(libvirt, inst))
+        self.update_power_state(
+            util_libvirt.extract_power_state(libvirt, inst))
         self.add_event('unpause', 'complete')
 
     def get_console_data(self, length):
@@ -871,11 +955,8 @@ class Instance(dbo):
             length, len(d))
         return d
 
-    def enqueue_delete(self):
-        self.enqueue_delete_remote(config.NODE_NAME)
-
     def enqueue_delete_remote(self, node):
-        db.enqueue(node, {
+        etcd.enqueue(node, {
             'tasks': [DeleteInstanceTask(self.uuid)]
         })
 
@@ -891,17 +972,13 @@ class Instance(dbo):
             self.state = self.STATE_ERROR
 
         self.error = error_msg
-        self.enqueue_delete()
+        self.enqueue_delete_remote(config.NODE_NAME)
 
 
 class Instances(dbo_iter):
     def __iter__(self):
         for _, i in etcd.get_all('instance', None):
-            i = Instance.from_db(i['uuid'])
-            if not i:
-                continue
-
-            out = self.apply_filters(i)
+            out = self.apply_filters(Instance(i))
             if out:
                 yield out
 
