@@ -1,3 +1,4 @@
+import copy
 import datetime
 import random
 from socket import error as socket_error
@@ -24,11 +25,18 @@ class WrongEventException(Exception):
     pass
 
 
+# NOTE(mikal): this is a hack to "turn up the knob" on how slow image
+# downloads can be while my ISP is congested because of COVID. We should
+# turn this back down as things improve.
+NETWORK_PATIENCE_FACTOR = 3
+
+
 class BaseTestCase(testtools.TestCase):
     def setUp(self):
         super(BaseTestCase, self).setUp()
 
-        self.system_client = apiclient.Client()
+        self.system_client = apiclient.Client(
+            async_strategy=apiclient.ASYNC_PAUSE)
 
     def _make_namespace(self, name, key):
         self._remove_namespace(name)
@@ -37,8 +45,8 @@ class BaseTestCase(testtools.TestCase):
         self.system_client.add_namespace_key(name, 'test', key)
         return apiclient.Client(
             base_url=self.system_client.base_url,
-            namespace=name,
-            key=key)
+            namespace=name, key=key,
+            async_strategy=apiclient.ASYNC_PAUSE)
 
     def _remove_namespace(self, name):
         ns = self.system_client.get_namespaces()
@@ -66,7 +74,7 @@ class BaseTestCase(testtools.TestCase):
 
     def _log_image_events(self, image_uuid):
         self._log_events(
-            image_uuid, self.system_client.get_image_events(image_uuid))
+            image_uuid, self.system_client.get_artifact_events(image_uuid))
 
     def _log_events(self, uuid, event_source):
         x = PrettyTable()
@@ -115,7 +123,7 @@ class BaseTestCase(testtools.TestCase):
         # morning it can take over 2 minutes to download a Ubuntu image.
         start_time = time.time()
         final = False
-        while time.time() - start_time < 300:
+        while time.time() - start_time < 5 * 60 * NETWORK_PATIENCE_FACTOR:
             i = self.system_client.get_instance(instance_uuid)
             if i['state'] in ['created', 'error']:
                 final = True
@@ -132,9 +140,9 @@ class BaseTestCase(testtools.TestCase):
                 'Instance %s was not created in a reasonable time (%s)'
                 % (instance_uuid, i))
 
-        # Once created, we shouldn't need more than five minutes for boot.
+        # Once created, we shouldn't need more than another 2 minutes for boot.
         start_time = time.time()
-        while time.time() - start_time < 600:
+        while time.time() - start_time < 2 * 60:
             for event in self.system_client.get_instance_events(instance_uuid):
                 if after and event['timestamp'] <= after:
                     continue
@@ -170,8 +178,8 @@ class BaseTestCase(testtools.TestCase):
     def _await_image_event(
             self, image_uuid, operation, message=None, after=None):
         start_time = time.time()
-        while time.time() - start_time < 900:
-            for event in self.system_client.get_image_events(image_uuid):
+        while time.time() - start_time < 5 * 60 * NETWORK_PATIENCE_FACTOR:
+            for event in self.system_client.get_artifact_events(image_uuid):
                 if after and event['timestamp'] <= after:
                     continue
 
@@ -191,16 +199,33 @@ class BaseTestCase(testtools.TestCase):
             'After time %s, image %s had no event type "%s" (waited 5 mins)'
             % (after, image_uuid, operation))
 
-    def _await_network_ready(self, network_uuid):
+    def _await_objects_ready(self, callback, items):
+        waiting_for = copy.copy(items)
         start_time = time.time()
-        while time.time() - start_time < 300:
-            n = self.system_client.get_network(network_uuid)
-            if n.get('state') == 'created':
-                return
-            time.sleep(5)
 
-        raise TimeoutException(
-            'Network %s never became ready (waited 5 mins)' % network_uuid)
+        while waiting_for:
+            for item in copy.copy(waiting_for):
+                n = callback(item)
+                if n.get('state') in ['created', 'deleted', 'error']:
+                    waiting_for.remove(item)
+
+            if waiting_for:
+                time.sleep(5)
+
+            if waiting_for and time.time() - start_time > 300:
+                raise TimeoutException(
+                    'Items %s never became ready (waited 5 mins)' % ', '.join(waiting_for))
+
+    def _await_networks_ready(self, network_uuids):
+        self._await_objects_ready(
+            self.system_client.get_network, network_uuids)
+
+    def _await_instances_ready(self, instance_uuids):
+        self._await_objects_ready(
+            self.system_client.get_instance, instance_uuids)
+
+        for instance_uuid in instance_uuids:
+            self.assertInstanceOk(instance_uuid)
 
     def _test_ping(self, instance_uuid, network_uuid, ip, expected, attempts=1):
         while attempts:
@@ -223,6 +248,10 @@ class BaseTestCase(testtools.TestCase):
             self.fail('Ping test failed. Expected %s != actual %s.\nout: %s\nerr: %s\n'
                       % (expected, actual, output['stdout'], output['stderr']))
 
+    def assertInstanceOk(self, instance_uuid):
+        inst = self.system_client.get_instance(instance_uuid)
+        self.assertTrue(inst['state'] == 'created')
+
 
 class BaseNamespacedTestCase(BaseTestCase):
     def __init__(self, *args, **kwargs):
@@ -241,22 +270,39 @@ class BaseNamespacedTestCase(BaseTestCase):
 
     def tearDown(self):
         super(BaseNamespacedTestCase, self).tearDown()
-        for inst in self.test_client.get_instances():
-            self.test_client.delete_instance(inst['uuid'])
+
+        non_blocking_client = apiclient.Client(
+            base_url=self.system_client.base_url,
+            namespace=self.namespace, key=self.namespace_key,
+            async_strategy=apiclient.ASYNC_CONTINUE)
+        for inst in non_blocking_client.get_instances():
+            non_blocking_client.delete_instance(inst['uuid'])
 
         start_time = time.time()
-        while time.time() - start_time < 300:
-            if not list(self.test_client.get_instances()):
+        while time.time() - start_time < 5 * 60:
+            if not list(non_blocking_client.get_instances()):
                 break
             time.sleep(5)
 
-        remaining_instances = list(self.test_client.get_instances())
+        remaining_instances = list(non_blocking_client.get_instances())
         if remaining_instances:
             self.fail('Failed to delete instances: %s'
                       % remaining_instances)
 
-        for net in self.test_client.get_networks():
-            self.test_client.delete_network(net['uuid'])
+        for net in non_blocking_client.get_networks():
+            non_blocking_client.delete_network(net['uuid'])
+
+        start_time = time.time()
+        while time.time() - start_time < 5 * 60:
+            if not list(non_blocking_client.get_networks()):
+                break
+            time.sleep(5)
+
+        remaining_networks = list(non_blocking_client.get_networks())
+        if remaining_networks:
+            self.fail('Failed to delete networks: %s'
+                      % remaining_networks)
+
         self._remove_namespace(self.namespace)
 
 
@@ -265,11 +311,11 @@ class TestDistroBoots(BaseNamespacedTestCase):
         super(TestDistroBoots, self).setUp()
         self.net = self.test_client.allocate_network(
             '192.168.242.0/24', True, True, '%s-net' % self.namespace)
-        self._await_network_ready(self.net['uuid'])
+        self._await_networks_ready([self.net['uuid']])
 
     def _test_distro_boot(self, base_image):
         inst = self.test_client.create_instance(
-            base_image, 1, 1024,
+            base_image.replace(':', '-').replace('.', ''), 1, 1024,
             [
                 {
                     'network_uuid': self.net['uuid']
@@ -298,12 +344,19 @@ class TestDistroBoots(BaseNamespacedTestCase):
 class LoggingSocket(object):
     ctrlc = '\x03'
 
-    def __init__(self, host, port):
+    def __init__(self, client, inst):
+        inst = client.get_instance(inst['uuid'])
+        if not inst['node']:
+            raise Exception('Host is None!')
+        if not inst['console_port']:
+            raise Exception('Port is None!')
+
         attempts = 5
         while attempts:
             try:
                 attempts -= 1
-                self.s = telnetlib.Telnet(host, port, 30)
+                self.s = telnetlib.Telnet(
+                    inst['node'], inst['console_port'], 30)
                 return
 
             except ConnectionRefusedError:
@@ -311,8 +364,8 @@ class LoggingSocket(object):
                 time.sleep(5)
 
         raise ConnectionRefusedError(
-            'Repeated telnet connection attempts failed: host=%s port=%s' %
-            (host, port))
+            'Repeated telnet connection attempts failed: host=%s port=%s'
+            % (inst['node'], inst['console_port']))
 
     def ensure_fresh(self):
         for d in [self.ctrlc, self.ctrlc, '\nexit\n', 'cirros\n', 'gocubsgo\n']:
